@@ -6,13 +6,18 @@ from itertools import izip
 
 import numpy as np
 
+import pandas.tslib as tslib
+import pandas.lib as lib
+import pandas.algos as _algos
+import pandas.index as _index
+from pandas.lib import Timestamp
+
 from pandas.core.common import ndtake
 from pandas.util.decorators import cache_readonly
 import pandas.core.common as com
-import pandas.lib as lib
-import pandas._algos as _algos
-from pandas.lib import Timestamp
 from pandas.util import py3compat
+from pandas.core.config import get_option
+
 
 __all__ = ['Index']
 
@@ -39,8 +44,9 @@ _o_dtype = np.dtype(object)
 
 
 def _shouldbe_timestamp(obj):
-    return (lib.is_datetime_array(obj) or lib.is_datetime64_array(obj)
-            or lib.is_timestamp_array(obj))
+    return (tslib.is_datetime_array(obj)
+            or tslib.is_datetime64_array(obj)
+            or tslib.is_timestamp_array(obj))
 
 
 class Index(np.ndarray):
@@ -77,9 +83,10 @@ class Index(np.ndarray):
     name = None
     asi8 = None
 
-    _engine_type = lib.ObjectEngine
+    _engine_type = _index.ObjectEngine
 
     def __new__(cls, data, dtype=None, copy=False, name=None):
+        from pandas.tseries.period import PeriodIndex
         if isinstance(data, np.ndarray):
             if issubclass(data.dtype.type, np.datetime64):
                 from pandas.tseries.index import DatetimeIndex
@@ -94,6 +101,8 @@ class Index(np.ndarray):
                     data = np.array(data, dtype=dtype, copy=copy)
                 except TypeError:
                     pass
+            elif isinstance(data, PeriodIndex):
+                return PeriodIndex(data, copy=copy, name=name)
 
             if issubclass(data.dtype.type, np.integer):
                 return Int64Index(data, copy=copy, name=name)
@@ -107,17 +116,18 @@ class Index(np.ndarray):
             subarr = com._asarray_tuplesafe(data, dtype=object)
 
         if dtype is None:
-            if _shouldbe_timestamp(subarr):
-                from pandas.tseries.index import DatetimeIndex
-                return DatetimeIndex(subarr, copy=copy, name=name)
+            inferred = lib.infer_dtype(subarr)
+            if inferred == 'integer':
+                return Int64Index(subarr.astype('i8'), name=name)
+            elif inferred != 'string':
+                if (inferred.startswith('datetime') or
+                        tslib.is_timestamp_array(subarr)):
+                    from pandas.tseries.index import DatetimeIndex
+                    return DatetimeIndex(subarr, copy=copy, name=name)
 
             if lib.is_period_array(subarr):
-                from pandas.tseries.period import PeriodIndex
                 return PeriodIndex(subarr, name=name)
 
-            if lib.is_integer_array(subarr):
-                return Int64Index(subarr.astype('i8'), name=name)
-
         subarr = subarr.view(cls)
         subarr.name = name
         return subarr
@@ -132,12 +142,49 @@ class Index(np.ndarray):
     def _shallow_copy(self):
         return self.view()
 
-    def __repr__(self):
+    def __str__(self):
+        """
+        Return a string representation for a particular Index
+
+        Invoked by str(df) in both py2/py3.
+        Yields Bytestring in Py2, Unicode String in py3.
+        """
+
         if py3compat.PY3:
-            prepr = com.pprint_thing(self)
+            return self.__unicode__()
+        return self.__bytes__()
+
+    def __bytes__(self):
+        """
+        Return a string representation for a particular Index
+
+        Invoked by bytes(df) in py3 only.
+        Yields a bytestring in both py2/py3.
+        """
+        encoding = com.get_option("display.encoding")
+        return self.__unicode__().encode(encoding, 'replace')
+
+    def __unicode__(self):
+        """
+        Return a string representation for a particular Index
+
+        Invoked by unicode(df) in py2 only. Yields a Unicode String in both py2/py3.
+        """
+        if len(self) > 6 and len(self) > np.get_printoptions()['threshold']:
+            data = self[:3].tolist() + ["..."] + self[-3:].tolist()
         else:
-            prepr = com.pprint_thing_encoded(self)
-        return 'Index(%s, dtype=%s)' % (prepr, self.dtype)
+            data = self
+
+        prepr = com.pprint_thing(data, escape_chars=('\t', '\r', '\n'))
+        return '%s(%s, dtype=%s)' % (type(self).__name__, prepr, self.dtype)
+
+    def __repr__(self):
+        """
+        Return a string representation for a particular Index
+
+        Yields Bytestring in Py2, Unicode String in py3.
+        """
+        return str(self)
 
     def astype(self, dtype):
         return Index(self.values.astype(dtype), name=self.name,
@@ -166,7 +213,7 @@ class Index(np.ndarray):
         """
         return list(self.values)
 
-    @property
+    @cache_readonly
     def dtype(self):
         return self.values.dtype
 
@@ -180,7 +227,9 @@ class Index(np.ndarray):
         return [self.name]
 
     def _set_names(self, values):
-        assert(len(values) == 1)
+        if len(values) != 1:
+            raise AssertionError('Length of new names must be 1, got %d'
+                                 % len(values))
         self.name = values[0]
 
     names = property(fset=_set_names, fget=_get_names)
@@ -205,15 +254,6 @@ class Index(np.ndarray):
             name = type(self).__name__
         return '%s: %s entries%s' % (name, len(self), index_summary)
 
-    def __str__(self):
-        try:
-            return np.array_repr(self.values)
-        except UnicodeError:
-            converted = u','.join(com.pprint_thing(x) for x in self.values)
-            result = u'%s([%s], dtype=''%s'')' % (type(self).__name__, converted,
-                                              str(self.values.dtype))
-            return com.console_encode(result)
-
     def _mpl_repr(self):
         # how to represent ourselves to matplotlib
         return self.values
@@ -255,7 +295,9 @@ class Index(np.ndarray):
 
     def _get_level_number(self, level):
         if not isinstance(level, int):
-            assert(level == self.name)
+            if level != self.name:
+                raise AssertionError('Level %s must be same as name (%s)'
+                                     % (level, self.name))
             level = 0
         return level
 
@@ -375,7 +417,7 @@ class Index(np.ndarray):
         taken = self.view(np.ndarray).take(indexer)
         return self._constructor(taken, name=self.name)
 
-    def format(self, name=False):
+    def format(self, name=False, formatter=None):
         """
         Render a string representation of the Index
         """
@@ -383,15 +425,20 @@ class Index(np.ndarray):
 
         header = []
         if name:
-            header.append(com.pprint_thing(self.name) if self.name is not None else '')
+            header.append(com.pprint_thing(self.name,
+                                           escape_chars=('\t', '\r', '\n'))
+                          if self.name is not None else '')
+
+        if formatter is not None:
+            return header + list(self.map(formatter))
 
         if self.is_all_dates:
             zero_time = time(0, 0)
             result = []
             for dt in self:
                 if dt.time() != zero_time or dt.tzinfo is not None:
-                    return header + ['%s' % x for x in self]
-                result.append('%d-%.2d-%.2d' % (dt.year, dt.month, dt.day))
+                    return header + [u'%s' % x for x in self]
+                result.append(u'%d-%.2d-%.2d' % (dt.year, dt.month, dt.day))
             return header + result
 
         values = self.values
@@ -400,7 +447,8 @@ class Index(np.ndarray):
             values = lib.maybe_convert_objects(values, safe=1)
 
         if values.dtype == np.object_:
-            result = [com.pprint_thing(x) for x in values]
+            result = [com.pprint_thing(x, escape_chars=('\t', '\r', '\n'))
+                      for x in values]
         else:
             result = _trim_front(format_array(values, None, justify='left'))
         return header + result
@@ -665,7 +713,7 @@ class Index(np.ndarray):
 
         Returns
         -------
-        loc : int
+        loc : int if unique index, possibly slice or mask if not
         """
         return self._engine.get_loc(key)
 
@@ -681,7 +729,7 @@ class Index(np.ndarray):
                 raise
 
             try:
-                return lib.get_value_box(series, key)
+                return tslib.get_value_box(series, key)
             except IndexError:
                 raise
             except TypeError:
@@ -705,6 +753,22 @@ class Index(np.ndarray):
         """
         self._engine.set_value(arr, key, value)
 
+    def get_level_values(self, level):
+        """
+        Return vector of label values for requested level, equal to the length
+        of the index
+
+        Parameters
+        ----------
+        level : int
+
+        Returns
+        -------
+        values : ndarray
+        """
+        num = self._get_level_number(level)
+        return self
+
     def get_indexer(self, target, method=None, limit=None):
         """
         Compute indexer and mask for new index given the current index. The
@@ -725,13 +789,12 @@ class Index(np.ndarray):
 
         Examples
         --------
-        >>> indexer, mask = index.get_indexer(new_index)
+        >>> indexer = index.get_indexer(new_index)
         >>> new_values = cur_values.take(indexer)
-        >>> new_values[-mask] = np.nan
 
         Returns
         -------
-        (indexer, mask) : (ndarray, ndarray)
+        indexer : ndarray
         """
         method = self._get_method(method)
         target = _ensure_index(target)
@@ -750,10 +813,12 @@ class Index(np.ndarray):
                             'objects')
 
         if method == 'pad':
-            assert(self.is_monotonic)
+            if not self.is_monotonic:
+                raise AssertionError('Must be monotonic for forward fill')
             indexer = self._engine.get_pad_indexer(target.values, limit)
         elif method == 'backfill':
-            assert(self.is_monotonic)
+            if not self.is_monotonic:
+                raise AssertionError('Must be monotonic for backward fill')
             indexer = self._engine.get_backfill_indexer(target.values, limit)
         elif method is None:
             indexer = self._engine.get_indexer(target.values)
@@ -1109,7 +1174,7 @@ class Index(np.ndarray):
         -------
         new_index : Index
         """
-        arr = np.delete(np.asarray(self), loc)
+        arr = np.delete(self.values, loc)
         return Index(arr)
 
     def insert(self, loc, item):
@@ -1160,7 +1225,7 @@ class Int64Index(Index):
     _inner_indexer = _algos.inner_join_indexer_int64
     _outer_indexer = _algos.outer_join_indexer_int64
 
-    _engine_type = lib.Int64Engine
+    _engine_type = _index.Int64Engine
 
     def __new__(cls, data, dtype=None, copy=False, name=None):
         if not isinstance(data, np.ndarray):
@@ -1197,7 +1262,7 @@ class Int64Index(Index):
     def _constructor(self):
         return Int64Index
 
-    @property
+    @cache_readonly
     def dtype(self):
         return np.dtype('int64')
 
@@ -1250,7 +1315,9 @@ class MultiIndex(Index):
     names = None
 
     def __new__(cls, levels=None, labels=None, sortorder=None, names=None):
-        assert(len(levels) == len(labels))
+        if len(levels) != len(labels):
+            raise AssertionError(
+                'Length of levels and labels must be the same')
         if len(levels) == 0:
             raise Exception('Must pass non-zero number of levels/labels')
 
@@ -1273,7 +1340,10 @@ class MultiIndex(Index):
         if names is None:
             subarr.names = [None] * subarr.nlevels
         else:
-            assert(len(names) == subarr.nlevels)
+            if len(names) != subarr.nlevels:
+                raise AssertionError(('Length of names must be same as level '
+                                      '(%d), got %d') % (subarr.nlevels))
+
             subarr.names = list(names)
 
         # set the name
@@ -1306,11 +1376,38 @@ class MultiIndex(Index):
         # hack for various methods
         return self.values
 
-    @property
+    @cache_readonly
     def dtype(self):
         return np.dtype('O')
 
-    def __repr__(self):
+    def __str__(self):
+        """
+        Return a string representation for a particular Index
+
+        Invoked by str(df) in both py2/py3.
+        Yields Bytestring in Py2, Unicode String in py3.
+        """
+
+        if py3compat.PY3:
+            return self.__unicode__()
+        return self.__bytes__()
+
+    def __bytes__(self):
+        """
+        Return a string representation for a particular Index
+
+        Invoked by bytes(df) in py3 only.
+        Yields a bytestring in both py2/py3.
+        """
+        encoding = com.get_option("display.encoding")
+        return self.__unicode__().encode(encoding, 'replace')
+
+    def __unicode__(self):
+        """
+        Return a string representation for a particular Index
+
+        Invoked by unicode(df) in py2 only. Yields a Unicode String in both py2/py3.
+        """
         output = 'MultiIndex\n%s'
 
         options = np.get_printoptions()
@@ -1322,14 +1419,19 @@ class MultiIndex(Index):
         else:
             values = self.values
 
-        summary = com.pprint_thing(values)
+        summary = com.pprint_thing(values, escape_chars=('\t', '\r', '\n'))
 
         np.set_printoptions(threshold=options['threshold'])
 
-        if py3compat.PY3:
-            return output % summary
-        else:
-            return com.console_encode(output % summary)
+        return output % summary
+
+    def __repr__(self):
+        """
+        Return a string representation for a particular Index
+
+        Yields Bytestring in Py2, Unicode String in py3.
+        """
+        return str(self)
 
     def __len__(self):
         return len(self.labels[0])
@@ -1382,7 +1484,7 @@ class MultiIndex(Index):
 
             values = []
             for lev, lab in zip(self.levels, self.labels):
-                taken = ndtake(lev.values, lab)
+                taken = com.take_1d(lev.values, lab)
                 # Need to box timestamps, etc.
                 if hasattr(lev, '_box_values'):
                     taken = lev._box_values(taken)
@@ -1444,7 +1546,7 @@ class MultiIndex(Index):
                 pass
 
             try:
-                return lib.get_value_at(series, key)
+                return _index.get_value_at(series, key)
             except IndexError:
                 raise
             except TypeError:
@@ -1472,22 +1574,23 @@ class MultiIndex(Index):
         values : ndarray
         """
         num = self._get_level_number(level)
-        unique_vals = self.levels[num].values
+        unique_vals = self.levels[num]  # .values
         labels = self.labels[num]
         return unique_vals.take(labels)
 
     def format(self, space=2, sparsify=None, adjoin=True, names=False,
-               na_rep='NaN'):
+               na_rep='NaN', formatter=None):
         if len(self) == 0:
             return []
 
         stringified_levels = []
         for lev, lab in zip(self.levels, self.labels):
             if len(lev) > 0:
-                formatted = lev.take(lab).format()
+                formatted = lev.take(lab).format(formatter=formatter)
             else:
                 # weird all NA case
-                formatted = [str(x) for x in com.take_1d(lev.values, lab)]
+                formatted = [com.pprint_thing(x, escape_chars=('\t', '\r', '\n'))
+                             for x in com.take_1d(lev.values, lab)]
             stringified_levels.append(formatted)
 
         result_levels = []
@@ -1495,14 +1598,14 @@ class MultiIndex(Index):
             level = []
 
             if names:
-                level.append(com.pprint_thing(name) if name is not None else '')
+                level.append(com.pprint_thing(name, escape_chars=('\t', '\r', '\n'))
+                             if name is not None else '')
 
             level.extend(np.array(lev, dtype=object))
             result_levels.append(level)
 
         if sparsify is None:
-            import pandas.core.format as fmt
-            sparsify = fmt.print_config.multi_sparse
+            sparsify = get_option("display.multi_sparse")
 
         if sparsify:
             # little bit of a kludge job for #1217
@@ -1687,12 +1790,17 @@ class MultiIndex(Index):
         -------
         appended : Index
         """
-        if isinstance(other, (list, tuple)):
-            to_concat = (self.values,) + tuple(k.values for k in other)
-        else:
-            to_concat = self.values, other.values
+        if not isinstance(other, (list, tuple)):
+            other = [other]
+
+        to_concat = (self.values,) + tuple(k.values for k in other)
         new_tuples = np.concatenate(to_concat)
-        return MultiIndex.from_tuples(new_tuples, names=self.names)
+
+        # if all(isinstance(x, MultiIndex) for x in other):
+        try:
+            return MultiIndex.from_tuples(new_tuples, names=self.names)
+        except:
+            return Index(new_tuples)
 
     def argsort(self, *args, **kwargs):
         return self.values.argsort()
@@ -1816,7 +1924,10 @@ class MultiIndex(Index):
         ----------
         """
         order = [self._get_level_number(i) for i in order]
-        assert(len(order) == self.nlevels)
+        if len(order) != self.nlevels:
+            raise AssertionError(('Length of order must be same as '
+                                  'number of levels (%d), got %d')
+                                 % (self.nlevels, len(order)))
         new_levels = [self.levels[i] for i in order]
         new_labels = [self.labels[i] for i in order]
         new_names = [self.names[i] for i in order]
@@ -1913,11 +2024,15 @@ class MultiIndex(Index):
         self_index = self._tuple_index
 
         if method == 'pad':
-            assert(self.is_unique and self.is_monotonic)
+            if not self.is_unique or not self.is_monotonic:
+                raise AssertionError(('Must be unique and monotonic to '
+                                      'use forward fill getting the indexer'))
             indexer = self_index._engine.get_pad_indexer(target_index,
                                                          limit=limit)
         elif method == 'backfill':
-            assert(self.is_unique and self.is_monotonic)
+            if not self.is_unique or not self.is_monotonic:
+                raise AssertionError(('Must be unique and monotonic to '
+                                      'use backward fill getting the indexer'))
             indexer = self_index._engine.get_backfill_indexer(target_index,
                                                               limit=limit)
         else:
@@ -2084,7 +2199,9 @@ class MultiIndex(Index):
             return new_index
 
         if isinstance(level, (tuple, list)):
-            assert(len(key) == len(level))
+            if len(key) != len(level):
+                raise AssertionError('Key for location must have same '
+                                     'length as number of levels')
             result = None
             for lev, k in zip(level, key):
                 loc, new_index = self.get_loc_level(k, level=lev)
@@ -2324,13 +2441,7 @@ class MultiIndex(Index):
                                           names=result_names)
 
     def _assert_can_do_setop(self, other):
-        if not isinstance(other, MultiIndex):
-            if len(other) == 0:
-                return True
-            raise TypeError('can only call with other hierarchical '
-                            'index objects')
-
-        assert(self.nlevels == other.nlevels)
+        pass
 
     def insert(self, loc, item):
         """
@@ -2346,9 +2457,12 @@ class MultiIndex(Index):
         -------
         new_index : Index
         """
-        if not isinstance(item, tuple) or len(item) != self.nlevels:
-            raise Exception("%s cannot be inserted in this MultiIndex"
-                            % str(item))
+        # Pad the key with empty strings if lower levels of the key
+        # aren't specified:
+        if not isinstance(item, tuple):
+            item = (item,) + ('',) * (self.nlevels - 1)
+        elif len(item) != self.nlevels:
+            raise ValueError('Passed item incompatible tuple length')
 
         new_levels = []
         new_labels = []
@@ -2439,15 +2553,13 @@ def _ensure_index(index_like):
         return Index(index_like, name=index_like.name)
 
     if isinstance(index_like, list):
-        klasses = (list, np.ndarray)
-        all_arrays = all(isinstance(x, klasses) for x in index_like)
+        # #2200 ?
+        converted, all_arrays = lib.clean_index_list(index_like)
 
-        if len(index_like) > 0 and all_arrays:
-            return MultiIndex.from_arrays(index_like)
+        if len(converted) > 0 and all_arrays:
+            return MultiIndex.from_arrays(converted)
         else:
-            # #2200 ?
-            index_like = [tuple(x) if isinstance(x, klasses) else x
-                          for x in index_like]
+            index_like = converted
 
     return Index(index_like)
 
@@ -2478,7 +2590,8 @@ def _get_distinct_indexes(indexes):
 
 
 def _union_indexes(indexes):
-    assert(len(indexes) > 0)
+    if len(indexes) == 0:
+        raise AssertionError('Must have at least 1 Index to union')
     if len(indexes) == 1:
         result = indexes[0]
         if isinstance(result, list):
@@ -2489,9 +2602,13 @@ def _union_indexes(indexes):
 
     if kind == 'special':
         result = indexes[0]
-        for other in indexes[1:]:
-            result = result.union(other)
-        return result
+
+        if hasattr(result, 'union_many'):
+            return result.union_many(indexes[1:])
+        else:
+            for other in indexes[1:]:
+                result = result.union(other)
+            return result
     elif kind == 'array':
         index = indexes[0]
         for other in indexes[1:]:
