@@ -20,19 +20,22 @@ from pandas.sparse.frame import SparseDataFrame
 import pandas.core.common as com
 
 import pandas.lib as lib
+import pandas.algos as algos
+import pandas.hashtable as _hash
 
 
 @Substitution('\nleft : DataFrame')
 @Appender(_merge_doc, indents=0)
 def merge(left, right, how='inner', on=None, left_on=None, right_on=None,
-          left_index=False, right_index=False, sort=True,
+          left_index=False, right_index=False, sort=False,
           suffixes=('_x', '_y'), copy=True):
     op = _MergeOperation(left, right, how=how, on=on, left_on=left_on,
                          right_on=right_on, left_index=left_index,
                          right_index=right_index, sort=sort, suffixes=suffixes,
                          copy=copy)
     return op.get_result()
-if __debug__: merge.__doc__ = _merge_doc % '\nleft : DataFrame'
+if __debug__:
+    merge.__doc__ = _merge_doc % '\nleft : DataFrame'
 
 
 class MergeError(Exception):
@@ -142,11 +145,8 @@ def ordered_merge(left, right, on=None, left_by=None, right_by=None,
         return _merger(left, right)
 
 
-
 # TODO: transformations??
 # TODO: only copy DataFrames when modification necessary
-
-
 class _MergeOperation(object):
     """
     Perform a database (SQL) merge operation between two DataFrame objects
@@ -214,8 +214,9 @@ class _MergeOperation(object):
                         continue
 
                     right_na_indexer = right_indexer.take(na_indexer)
-                    key_col.put(na_indexer, com.take_1d(self.right_join_keys[i],
-                                                        right_na_indexer))
+                    key_col.put(
+                        na_indexer, com.take_1d(self.right_join_keys[i],
+                                                right_na_indexer))
                 elif name in self.right and right_indexer is not None:
                     na_indexer = (right_indexer == -1).nonzero()[0]
                     if len(na_indexer) == 0:
@@ -366,7 +367,7 @@ class _MergeOperation(object):
     def _validate_specification(self):
         # Hm, any way to make this logic less complicated??
         if (self.on is None and self.left_on is None
-            and self.right_on is None):
+                and self.right_on is None):
 
             if self.left_index and self.right_index:
                 self.left_on, self.right_on = (), ()
@@ -377,15 +378,24 @@ class _MergeOperation(object):
                 if self.left_on is None:
                     raise MergeError('Must pass left_on or left_index=True')
             else:
+                if not self.left.columns.is_unique:
+                    raise MergeError("Left data columns not unique: %s"
+                                     % repr(self.left.columns))
+
+                if not self.right.columns.is_unique:
+                    raise MergeError("Right data columns not unique: %s"
+                                     % repr(self.right.columns))
+
                 # use the common columns
-                common_cols = self.left.columns.intersection(self.right.columns)
+                common_cols = self.left.columns.intersection(
+                    self.right.columns)
                 if len(common_cols) == 0:
                     raise MergeError('No common columns to perform merge on')
                 self.left_on = self.right_on = common_cols
         elif self.on is not None:
             if self.left_on is not None or self.right_on is not None:
                 raise MergeError('Can only pass on OR left_on and '
-                                'right_on')
+                                 'right_on')
             self.left_on = self.right_on = self.on
         elif self.left_on is not None:
             n = len(self.left_on)
@@ -423,18 +433,20 @@ def _get_join_indexers(left_keys, right_keys, sort=False, how='inner'):
         right_labels.append(rlab)
         group_sizes.append(count)
 
-    left_group_key = get_group_index(left_labels, group_sizes)
-    right_group_key = get_group_index(right_labels, group_sizes)
-
     max_groups = 1L
     for x in group_sizes:
         max_groups *= long(x)
 
     if max_groups > 2 ** 63:  # pragma: no cover
-        raise MergeError('Combinatorial explosion! (boom)')
+        left_group_key, right_group_key, max_groups = \
+            _factorize_keys(lib.fast_zip(left_labels),
+                            lib.fast_zip(right_labels))
+    else:
+        left_group_key = get_group_index(left_labels, group_sizes)
+        right_group_key = get_group_index(right_labels, group_sizes)
 
-    left_group_key, right_group_key, max_groups = \
-        _factorize_keys(left_group_key, right_group_key, sort=sort)
+        left_group_key, right_group_key, max_groups = \
+            _factorize_keys(left_group_key, right_group_key, sort=sort)
 
     join_func = _join_functions[how]
     return join_func(left_group_key, right_group_key, max_groups)
@@ -464,8 +476,8 @@ class _OrderedMerge(_MergeOperation):
         ldata, rdata = self._get_merge_data()
 
         if self.fill_method == 'ffill':
-            left_join_indexer = lib.ffill_indexer(left_indexer)
-            right_join_indexer = lib.ffill_indexer(right_indexer)
+            left_join_indexer = algos.ffill_indexer(left_indexer)
+            right_join_indexer = algos.ffill_indexer(right_indexer)
         else:
             left_join_indexer = left_indexer
             right_join_indexer = right_indexer
@@ -498,9 +510,9 @@ def _get_multiindex_indexer(join_keys, index, sort=False):
                         sort=False)
 
     left_indexer, right_indexer = \
-        lib.left_outer_join(com._ensure_int64(left_group_key),
-                            com._ensure_int64(right_group_key),
-                            max_groups, sort=False)
+        algos.left_outer_join(com._ensure_int64(left_group_key),
+                              com._ensure_int64(right_group_key),
+                              max_groups, sort=False)
 
     return left_indexer, right_indexer
 
@@ -509,9 +521,9 @@ def _get_single_indexer(join_key, index, sort=False):
     left_key, right_key, count = _factorize_keys(join_key, index, sort=sort)
 
     left_indexer, right_indexer = \
-        lib.left_outer_join(com._ensure_int64(left_key),
-                            com._ensure_int64(right_key),
-                            count, sort=sort)
+        algos.left_outer_join(com._ensure_int64(left_key),
+                              com._ensure_int64(right_key),
+                              count, sort=sort)
 
     return left_indexer, right_indexer
 
@@ -543,40 +555,43 @@ def _left_join_on_index(left_ax, right_ax, join_keys, sort=False):
 
 
 def _right_outer_join(x, y, max_groups):
-    right_indexer, left_indexer = lib.left_outer_join(y, x, max_groups)
+    right_indexer, left_indexer = algos.left_outer_join(y, x, max_groups)
     return left_indexer, right_indexer
 
 _join_functions = {
-    'inner': lib.inner_join,
-    'left': lib.left_outer_join,
+    'inner': algos.inner_join,
+    'left': algos.left_outer_join,
     'right': _right_outer_join,
-    'outer': lib.full_outer_join,
+    'outer': algos.full_outer_join,
 }
 
 
 def _factorize_keys(lk, rk, sort=True):
     if com._is_int_or_datetime_dtype(lk) and com._is_int_or_datetime_dtype(rk):
-        klass = lib.Int64Factorizer
+        klass = _hash.Int64Factorizer
         lk = com._ensure_int64(lk)
         rk = com._ensure_int64(rk)
     else:
-        klass = lib.Factorizer
+        klass = _hash.Factorizer
         lk = com._ensure_object(lk)
         rk = com._ensure_object(rk)
 
     rizer = klass(max(len(lk), len(rk)))
 
-    llab, _ = rizer.factorize(lk)
-    rlab, _ = rizer.factorize(rk)
+    llab = rizer.factorize(lk)
+    rlab = rizer.factorize(rk)
 
     count = rizer.get_count()
 
     if sort:
-        llab, rlab = _sort_labels(rizer.uniques, llab, rlab)
+        uniques = rizer.uniques.to_array()
+        llab, rlab = _sort_labels(uniques, llab, rlab)
 
     # NA group
-    lmask = llab == -1; lany = lmask.any()
-    rmask = rlab == -1; rany = rmask.any()
+    lmask = llab == -1
+    lany = lmask.any()
+    rmask = rlab == -1
+    rany = rmask.any()
 
     if lany or rany:
         if lany:
@@ -698,7 +713,7 @@ class _BlockJoinOperation(object):
 
         sofar = 0
         for unit, blk in merge_chunks:
-            out_chunk = out[sofar : sofar + len(blk)]
+            out_chunk = out[sofar: sofar + len(blk)]
 
             if unit.indexer is None:
             # is this really faster than assigning to arr.flat?
@@ -859,10 +874,11 @@ def concat(objs, axis=0, join='outer', join_axes=None, ignore_index=False,
     names : list, default None
         Names for the levels in the resulting hierarchical index
     ignore_index : boolean, default False
-        If True, do not use the index values on the concatenation axis. The
+        If True, do not use the index values along the concatenation axis. The
         resulting axis will be labeled 0, ..., n - 1. This is useful if you are
         concatenating objects where the concatenation axis does not have
-        meaningful indexing information.
+        meaningful indexing information. Note the the index values on the other
+        axes are still respected in the join.
 
     Notes
     -----
@@ -971,12 +987,12 @@ class _Concatenator(object):
             data = data.consolidate()
             type_map = dict((type(blk), blk) for blk in data.blocks)
             blockmaps.append(type_map)
-        return blockmaps
+        return blockmaps, reindexed_data
 
     def _get_concatenated_data(self):
         try:
             # need to conform to same other (joined) axes for block join
-            blockmaps = self._prepare_blocks()
+            blockmaps, rdata = self._prepare_blocks()
             kinds = _get_all_block_kinds(blockmaps)
 
             new_blocks = []
@@ -1000,7 +1016,7 @@ class _Concatenator(object):
 
             new_data = {}
             for item in self.new_axes[0]:
-                new_data[item] = self._concat_single_item(item)
+                new_data[item] = self._concat_single_item(rdata, item)
 
         return new_data
 
@@ -1024,7 +1040,7 @@ class _Concatenator(object):
 
     def _concat_blocks(self, blocks):
         values_list = [b.values for b in blocks if b is not None]
-        concat_values = com._concat_compat(values_list)
+        concat_values = com._concat_compat(values_list, axis=self.axis)
 
         if self.axis > 0:
             # Not safe to remove this check, need to profile
@@ -1034,7 +1050,7 @@ class _Concatenator(object):
             return make_block(concat_values, blocks[0].items, self.new_axes[0])
         else:
             offsets = np.r_[0, np.cumsum([len(x._data.axes[0]) for
-                                            x in self.objs])]
+                                          x in self.objs])]
             indexer = np.concatenate([offsets[i] + b.ref_locs
                                       for i, b in enumerate(blocks)
                                       if b is not None])
@@ -1049,15 +1065,20 @@ class _Concatenator(object):
 
             return make_block(concat_values, concat_items, self.new_axes[0])
 
-    def _concat_single_item(self, item):
+    def _concat_single_item(self, objs, item):
         all_values = []
         dtypes = set()
-        for obj in self.objs:
-            try:
-                values = obj._data.get(item)
+
+        # le sigh
+        if isinstance(self.objs[0], SparseDataFrame):
+            objs = [x._data for x in self.objs]
+
+        for data, orig in zip(objs, self.objs):
+            if item in orig:
+                values = data.get(item)
                 dtypes.add(values.dtype)
                 all_values.append(values)
-            except KeyError:
+            else:
                 all_values.append(None)
 
         # this stinks
@@ -1071,9 +1092,9 @@ class _Concatenator(object):
             empty_dtype = np.float64
 
         to_concat = []
-        for obj, item_values in zip(self.objs, all_values):
+        for obj, item_values in zip(objs, all_values):
             if item_values is None:
-                shape = obj._data.shape[1:]
+                shape = obj.shape[1:]
                 missing_arr = np.empty(shape, dtype=empty_dtype)
                 missing_arr.fill(np.nan)
                 to_concat.append(missing_arr)
@@ -1131,7 +1152,13 @@ class _Concatenator(object):
             if self.axis == 0:
                 indexes = [x.index for x in self.objs]
             elif self.keys is None:
-                return Index(np.arange(len(self.objs)))
+                names = []
+                for x in self.objs:
+                    if x.name is not None:
+                        names.append(x.name)
+                    else:
+                        return Index(np.arange(len(self.objs)))
+                return Index(names)
             else:
                 return _ensure_index(self.keys)
         else:
@@ -1161,7 +1188,7 @@ def _concat_indexes(indexes):
 
 def _make_concat_multiindex(indexes, keys, levels=None, names=None):
     if ((levels is None and isinstance(keys[0], tuple)) or
-        (levels is not None and len(levels) > 1)):
+            (levels is not None and len(levels) > 1)):
         zipped = zip(*keys)
         if names is None:
             names = [None] * len(zipped)
